#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: Apache-2.0

Application entrypoint. Creates Flask app and swagger docs, adds endpoints
"""

__author__ = "Carmen Tawalika, Lina Krisztian"
__copyright__ = "2025-present mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


from flask import Flask
from flask_cors import CORS
from flask_restful_swagger_2 import Api

from actinia_cloudevent_plugin.endpoints import create_endpoints
from actinia_cloudevent_plugin.resources.logging import log

flask_app = Flask(__name__)
# allows endpoints with and without trailing slashes
flask_app.url_map.strict_slashes = False
# setting, to keep order of sorted dictionary, passed to jsonify()
flask_app.json.sort_keys = False
CORS(flask_app)


API_VERSION = "v1"

URL_PREFIX = f"/api/{API_VERSION}"

apidoc = Api(
    flask_app,
    title="actinia-cloudevent-plugin",
    # prefix=URL_PREFIX,
    api_version=API_VERSION,
    api_spec_url=f"{URL_PREFIX}/swagger",
    schemes=["https", "http"],
    consumes=["application/json"],
    description="""Receives cloudevent,
                   transforms it to an actinia process chain
                   and returns cloudevent back.
                   """,
)

create_endpoints(flask_api=apidoc)

if __name__ == "__main__":
    # call this for development only with:
    # `python3 -m actinia_cloudevent_plugin.main`
    log.debug("starting app in development mode...")
    # ignore ruff: S201 :Use of `debug=True` in Flask app detected
    flask_app.run(debug=True, use_reloader=False)  # noqa: S201
    # for production environent use application in wsgi.py



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2018-2024 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Response models
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2022 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


from typing import ClassVar

from flask_restful_swagger_2 import Schema


class SimpleStatusCodeResponseModel(Schema):
    """Simple response schema to inform about status."""

    type: str = "object"
    properties: ClassVar[dict] = {
        "status": {
            "type": "number",
            "description": "The status code of the request.",
        },
        "message": {
            "type": "string",
            "description": "A short message to describes the status",
        },
    }
    required: ClassVar[list[str]] = ["status", "message"]


simple_response_example = SimpleStatusCodeResponseModel(
    status=200,
    message="success",
)
SimpleStatusCodeResponseModel.example = simple_response_example



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Hello World class
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2025 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

import json

import requests
from cloudevents.conversion import to_binary
from cloudevents.http import CloudEvent
from flask import jsonify, make_response, request
from flask_restful_swagger_2 import Resource, swagger

from actinia_cloudevent_plugin.apidocs import hooks
from actinia_cloudevent_plugin.model.response_models import (
    SimpleStatusCodeResponseModel,
)
from actinia_cloudevent_plugin.resources.config import EVENTRECEIVER


class Hooks(Resource):
    """Webhook handling."""

    def get(self, source_name):
        """Cloudevent get method: not allowed response."""
        _source_name = source_name
        res = jsonify(
            SimpleStatusCodeResponseModel(
                status=405,
                message="Method Not Allowed",
            ),
        )
        return make_response(res, 405)

    def head(self, source_name):
        """Cloudevent head method: return empty response."""
        _source_name = source_name
        return make_response("", 200)

    @swagger.doc(hooks.describe_hooks_post_docs)
    def post(self, source_name) -> SimpleStatusCodeResponseModel:
        """Translate actinia webhook call to cloudevent.

        This method is called by HTTP POST actinia-core webhook
        """
        # only actinia as source supported so far
        if source_name != "actinia":
            return make_response(
                jsonify(
                    SimpleStatusCodeResponseModel(
                        status=400,
                        message="Bad Request: Source name not 'actinia'",
                    ),
                ),
                400,
            )

        postbody = request.get_json(force=True)

        if type(postbody) is dict:
            postbody = json.dumps(postbody)
        elif not isinstance(postbody, str):
            postbody = str(postbody)

        resp = json.loads(postbody)
        if "resource_id" not in resp:
            return make_response(
                jsonify(
                    SimpleStatusCodeResponseModel(
                        status=400,
                        message="Bad Request: No resource_id found in request",
                    ),
                ),
                400,
            )

        # TODO: define when to send cloudevent
        status = resp["status"]
        if status == "finished":
            # TODO send cloudevent
            pass
        terminate_status = ["finished", "error", "terminated"]
        if status in terminate_status:
            # TODO send cloudevent
            pass

        # TODO: move to common function from core.cloudevents
        url = EVENTRECEIVER.url
        try:
            attributes = {
                "specversion": "1.0",
                "source": "/actinia-cloudevent-plugin",
                "type": "com.mundialis.actinia.process.status",
                "subject": "nc_spm_08/PERMANENT",
                "datacontenttype": "application/json",
            }
            data = {"actinia_job": resp}
            event = CloudEvent(attributes, data)
            headers, body = to_binary(event)
            requests.post(url, headers=headers, data=body)
        except ConnectionError as e:
            return f"Connection ERROR when returning cloudevent: {e}"
        except Exception() as e:
            return f"ERROR when returning cloudevent: {e}"

        res = jsonify(
            SimpleStatusCodeResponseModel(
                status=200,
                message="Thank you for your update",
            ),
        )
        return make_response(res, 200)



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Hello World class
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Lina Krisztian"
__copyright__ = "Copyright 2025 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

from flask import jsonify, make_response
from flask_restful_swagger_2 import Resource, swagger
from requests.exceptions import ConnectionError  # noqa: A004

from actinia_cloudevent_plugin.apidocs import cloudevent
from actinia_cloudevent_plugin.core.cloudevents import (
    receive_cloud_event,
    send_binary_cloud_event,
    # send_structured_cloud_event,
    start_actinia_job,
)
from actinia_cloudevent_plugin.model.response_models import (
    SimpleStatusCodeResponseModel,
)
from actinia_cloudevent_plugin.resources.config import ACTINIA, EVENTRECEIVER
from actinia_cloudevent_plugin.resources.logging import log


class Cloudevent(Resource):
    """Cloudevent handling."""

    def __init__(self) -> None:
        """Cloudevent class initialisation."""
        self.msg = (
            "Received event <EVENT1> and returned event <EVENT2>"
            " with actinia-job <ACTINIA_JOB>."
        )

    def get(self):
        """Cloudevent get method: not allowed response."""
        res = jsonify(
            SimpleStatusCodeResponseModel(
                status=405,
                message="Method Not Allowed",
            ),
        )
        return make_response(res, 405)

    @swagger.doc(cloudevent.describe_cloudevent_post_docs)
    def post(self) -> SimpleStatusCodeResponseModel:
        """Cloudevent post method with cloudevent from postbody.

        Receives cloudevent, transforms to process chain (pc),
        sends pc to actinia + start process,
        and returns cloudevent with queue name.
        """
        # Transform postbody to cloudevent
        event_received = receive_cloud_event()
        # With received process chain start actinia process + return cloudevent
        actinia_resp = start_actinia_job(event_received)
        queue_name = actinia_resp["queue"]
        resource_id = actinia_resp["resource_id"]

        try:
            if queue_name == "local":
                # Nothing to do here, the configured actinia-core
                # instance is using a local queue, meaning that the
                # job is processed directly.
                log.info("No need to start actinia-worker")
            else:
                url = ACTINIA.worker_http_launcher_url
                if "{{ queue_name }}" in url:
                    url.replace("{{ queue_name }}", queue_name)
                new_event = send_binary_cloud_event(
                    event_received,
                    queue_name,
                    url,
                )

            # Send event to configured broker
            url = EVENTRECEIVER.url
            # TODO: binary or structured cloud event?
            new_event = send_binary_cloud_event(
                event_received,
                actinia_resp["queue"],
                url,
            )
            response = {
                "status": 201,
                "message": self.msg.replace("<EVENT1>", event_received["id"])
                .replace("<EVENT2>", new_event["id"])
                .replace("<ACTINIA_JOB>", queue_name),
                "actinia_queue_name": queue_name,
                "actinia_job": resource_id,
            }
            return make_response(jsonify(response), 201)

        except ConnectionError as e:
            return f"Connection ERROR when returning cloudevent: {e}"



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Add endpoints to flask app with endpoint definitions and routes
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Carmen Tawalika, Anika Weinmann, Lina Krisztian"
__copyright__ = "Copyright 2022-2024 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


from flask_restful_swagger_2 import Api

from actinia_cloudevent_plugin.api.cloudevents import Cloudevent
from actinia_cloudevent_plugin.api.hooks import Hooks
from actinia_cloudevent_plugin.api.version import Version


# endpoints loaded if run as actinia-core plugin as well as standalone app
def create_endpoints(flask_api: Api) -> None:
    """Create plugin endpoints."""
    apidoc = flask_api
    apidoc.add_resource(Cloudevent, "/")
    apidoc.add_resource(Hooks, "/hooks/<string:source_name>")
    apidoc.add_resource(Version, "/version")



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Apidocs for webhook endpoint
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2025 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


from actinia_cloudevent_plugin.model.response_models import (
    SimpleStatusCodeResponseModel,
)

describe_hooks_post_docs = {
    # "summary" is taken from the description of the get method
    "tags": ["cloudevent"],
    "description": (
        "Receives webhook with status update e.g. from actinia-core,"
        " transforms to cloudevent and sends it to configurable endpoint."
    ),
    "responses": {
        "200": {
            "description": (
                "This response returns a cloud event, "
                "generated from actinia-core status"
            ),
            "schema": SimpleStatusCodeResponseModel,
        },
        "400": {
            "description": "This response returns an error message",
            "schema": SimpleStatusCodeResponseModel,
        },
    },
}



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: Apache-2.0

Configuration file
"""

__author__ = "Carmen Tawalika, Lina Krisztian"
__copyright__ = "2018-2025 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


import configparser
from pathlib import Path

# config can be overwritten by mounting *.ini files into folders inside
# the config folder.
DEFAULT_CONFIG_PATH = "config"
CONFIG_FILES = [
    str(f) for f in Path(DEFAULT_CONFIG_PATH).glob("**/*.ini") if f.is_file()
]
GENERATED_CONFIG = DEFAULT_CONFIG_PATH + "/actinia-cloudevent-plugin.cfg"


class ACTINIA:
    """Default config for actinia processing."""

    enqueue_job_base_url = "http://localhost:8088/"
    use_actinia_modules = True
    user = "actinia-gdi"
    password = "actinia-gdi"
    worker_http_launcher_url = "http://localhost:8000/launch"


class EVENTRECEIVER:
    """Default config for cloudevent receiver."""

    url = "http://localhost:3000/"


class LOGCONFIG:
    """Default config for logging."""

    logfile = "actinia-cloudevent-plugin.log"
    level = "INFO"
    type = "stdout"


class Configfile:
    """Configuration file."""

    def __init__(self) -> None:
        """Overwrite config classes.

        Will overwrite the config classes above when config files
        named DEFAULT_CONFIG_PATH/**/*.ini exist.
        On first import of the module it is initialized.
        """
        config = configparser.ConfigParser()
        config.read(CONFIG_FILES)
        if len(config) <= 1:
            print("Could not find any config file, using default values.")
            return
        print("Loading config files: " + str(CONFIG_FILES) + " ...")

        with open(  # noqa: PTH123
            GENERATED_CONFIG,
            "w",
            encoding="utf-8",
        ) as configfile:
            config.write(configfile)
        print("Configuration written to " + GENERATED_CONFIG)

        # ACTINIA
        if config.has_section("ACTINIA"):
            if config.has_option("ACTINIA", "enqueue_job_base_url"):
                ACTINIA.enqueue_job_base_url = config.get(
                    "ACTINIA",
                    "enqueue_job_base_url",
                )
            if config.has_option("ACTINIA", "use_actinia_modules"):
                ACTINIA.use_actinia_modules = config.getboolean(
                    "ACTINIA",
                    "use_actinia_modules",
                )
            if config.has_option("ACTINIA", "user"):
                ACTINIA.user = config.get("ACTINIA", "user")
            if config.has_option("ACTINIA", "password"):
                ACTINIA.password = config.get("ACTINIA", "password")
            if config.has_option("ACTINIA", "worker_http_launcher_url"):
                ACTINIA.worker_http_launcher_url = config.get(
                    "ACTINIA",
                    "worker_http_launcher_url",
                )
        # LOGGING
        if config.has_section("LOGCONFIG"):
            if config.has_option("LOGCONFIG", "logfile"):
                LOGCONFIG.logfile = config.get("LOGCONFIG", "logfile")
            if config.has_option("LOGCONFIG", "level"):
                LOGCONFIG.level = config.get("LOGCONFIG", "level")
            if config.has_option("LOGCONFIG", "type"):
                LOGCONFIG.type = config.get("LOGCONFIG", "type")

        # EVENTRECEIVER
        if config.has_section("EVENTRECEIVER"):
            if config.has_option("EVENTRECEIVER", "url"):
                EVENTRECEIVER.url = config.get("EVENTRECEIVER", "url")


init = Configfile()



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

actinia plugin initalization
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Carmen Tawalika, Anika Weinmann"
__copyright__ = "Copyright 2022 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


import importlib.metadata

try:
    # Change here if project is renamed and does not equal the package name
    DIST_NAME = __name__
    __version__ = importlib.metadata.version(DIST_NAME)
except Exception():
    __version__ = "unknown"



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Version information
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2025 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

import os
import sys
from importlib import metadata

from flask import jsonify, make_response
from flask_restful_swagger_2 import Resource


class Version(Resource):
    """Version information."""

    def get(self):
        """Get version information."""
        info = {}

        plugin_name = __package__.split(".")[0]
        info["version"] = metadata.version(plugin_name)

        python_version = sys.version.replace("\n", "- ")
        info["python_version"] = python_version

        env_name = "RUNNING_SINCE"
        if env_name in os.environ:
            info["running_since"] = os.environ[env_name]
        else:
            info["running_since"] = "n/a"

        return make_response(jsonify(info), 200)



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later
"""

from __future__ import annotations

__license__ = "GPL-3.0-or-later"
__author__ = "Carmen Tawalika, Sören Gebbert"
__copyright__ = "Copyright 2018-2022 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

import unittest

from actinia_cloudevent_plugin.main import flask_app


class TestCase(unittest.TestCase):
    """Test case class."""

    URL_PREFIX = "http://localhost:5000/api/v1"

    def setUp(self) -> None:
        """Overwrite method setUp from unittest.TestCase class."""
        self.app_context = flask_app.app_context()
        self.app_context.push()
        # from http://flask.pocoo.org/docs/0.12/api/#flask.Flask.test_client:
        # Note that if you are testing for assertions or exceptions in your
        # application code, you must set app.testing = True in order for the
        # exceptions to propagate to the test client.  Otherwise, the exception
        # will be handled by the application (not visible to the test client)
        # and the only indication of an AssertionError or other exception will
        # be a 500 status code response to the test client.
        flask_app.testing = True
        self.app = flask_app.test_client()

    def tearDown(self) -> None:
        """Overwrite method tearDown from unittest.TestCase class."""
        self.app_context.pop()



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2024 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Plugin setup file
"""

from setuptools import setup

if __name__ == "__main__":
    setup()



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: Apache-2.0

Logging interface
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2025, mundialis"
__maintainer__ = ""


import logging
from datetime import datetime, timezone
from logging import FileHandler

from colorlog import ColoredFormatter
from pythonjsonlogger import json

from actinia_cloudevent_plugin.resources.config import LOGCONFIG

# Notice: do not call logging.warning (will create new logger for ever)
# logging.warning("called actinia_cloudevent_plugin logger after 1")

log = logging.getLogger("actinia-cloudevent-plugin")
werkzeugLog = logging.getLogger("werkzeug")
gunicornLog = logging.getLogger("gunicorn")


def set_log_format(veto=None):
    """Set format of logs."""
    logformat = ""
    if LOGCONFIG.type == "json" and not veto:
        logformat = CustomJsonFormatter(
            "%(time) %(level) %(component)"
            "%(module) %(message) %(pathname)"
            "%(lineno) %(processName)"
            "%(threadName)",
        )
    else:
        logformat = ColoredFormatter(
            "%(log_color)s[%(asctime)s] %(levelname)-10s: %(name)s.%(module)-"
            "10s -%(message)s [in %(pathname)s:%(lineno)d]%(reset)s",
        )
    return logformat


def set_log_handler(logger, logtype, logformat) -> None:
    """Set handling of logs."""
    if logtype == "stdout":
        handler = logging.StreamHandler()
    elif logtype == "file":
        # For readability, json is never written to file
        handler = FileHandler(LOGCONFIG.logfile)
    handler.setFormatter(logformat)
    logger.addHandler(handler)


class CustomJsonFormatter(json.JsonFormatter):
    """Customized formatting of logs as json."""

    def add_fields(self, log_record, record, message_dict) -> None:
        """Add fiels for json log."""
        super(CustomJsonFormatter, self).add_fields(  # noqa: UP008
            log_record,
            record,
            message_dict,
        )

        # (Pdb) dir(record)
        # ... 'args', 'created', 'exc_info', 'exc_text', 'filename', 'funcName'
        # ,'getMessage', 'levelname', 'levelno', 'lineno', 'message', 'module',
        # 'msecs', 'msg', 'name', 'pathname', 'process', 'processName',
        # 'relativeCreated', 'stack_info', 'thread', 'threadName']
        now = datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%S.%fZ")
        log_record["time"] = now
        log_record["level"] = record.levelname
        log_record["component"] = record.name


def create_logger() -> None:
    """Create logger, set level and define format."""
    log.setLevel(getattr(logging, LOGCONFIG.level))
    fileformat = set_log_format("veto")
    stdoutformat = set_log_format()
    set_log_handler(log, "file", fileformat)
    set_log_handler(log, "stdout", stdoutformat)


def create_werkzeug_logger() -> None:
    """Create werkzeug-logger, set level and define format."""
    werkzeugLog.setLevel(getattr(logging, LOGCONFIG.level))
    fileformat = set_log_format("veto")
    stdoutformat = set_log_format()
    set_log_handler(werkzeugLog, "file", fileformat)
    set_log_handler(werkzeugLog, "stdout", stdoutformat)


def create_gunicorn_logger() -> None:
    """Create gunicorn-logger, set level and define format."""
    gunicornLog.setLevel(getattr(logging, LOGCONFIG.level))
    fileformat = set_log_format("veto")
    stdoutformat = set_log_format()
    set_log_handler(gunicornLog, "file", fileformat)
    set_log_handler(gunicornLog, "stdout", stdoutformat)
    # gunicorn already has a lot of children logger, e.g gunicorn.http,
    # gunicorn.access. These lines deactivate their default handlers.
    # pylint: disable=E1101
    for name in logging.root.manager.loggerDict:
        if "gunicorn." in name:
            logging.getLogger(name).propagate = True
            logging.getLogger(name).handlers = []


create_logger()
create_werkzeug_logger()
create_gunicorn_logger()



#!/usr/bin/env python
"""SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG.

SPDX-License-Identifier: GPL-3.0-or-later

Example core functionality
"""

__license__ = "GPL-3.0-or-later"
__author__ = "Lina Krisztian"
__copyright__ = "Copyright 2025 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


import json

import requests
from cloudevents.conversion import to_binary, to_structured
from cloudevents.http import CloudEvent, from_http
from flask import request
from requests.auth import HTTPBasicAuth

from actinia_cloudevent_plugin.resources.config import ACTINIA, EVENTRECEIVER


def receive_cloud_event():
    """Return cloudevent from postpody."""
    # Parses CloudEvent 'data' and 'headers' into a CloudEvent.
    event = from_http(request.headers, request.get_data())

    # ? TODO
    # eventually Filter the event (see example below)
    event_type = event["type"]
    if event_type == "com.example.object.created":
        print("Object created event received!")

    return event


def start_actinia_job(event) -> str:
    """Return actinia response for process chain of event."""
    pc = event.get_data()
    # NOTE: as standalone app -> consider for queue name creation

    # TODO: Define ce attribute for possible mapset.
    # Also in actiniaproject divided by "/" or "."?
    project = event.get_attributes().get("actiniaproject")
    mapset = None
    if "." in project:
        project = project.split(".")[0]
        mapset = project.split(".")[1]

    url = f"{ACTINIA.enqueue_job_base_url}/projects/{project}/"
    if not mapset:
        # emphemeral processing
        if ACTINIA.use_actinia_modules:
            url += "processing_export"
        else:
            url += "processing_async_export"
    # persistent processing
    elif ACTINIA.use_actinia_modules:
        url += f"mapsets/{mapset}/processing"
    else:
        url += f"mapsets/{mapset}/processing_async/"

    postkwargs = dict()
    postkwargs["headers"] = {"content-type": "application/json; charset=utf-8"}
    postkwargs["auth"] = HTTPBasicAuth(ACTINIA.user, ACTINIA.password)
    postkwargs["data"] = json.dumps(pc)

    resp = requests.post(
        url,
        **postkwargs,
    )

    # Part of resp:
    # 'message' = 'Resource accepted'
    # 'queue' = 'job_queue_resource_id-cddae7bb-b4fa-4249-aec4-2a646946ff36'
    # 'resource_id' = 'resource_id-cddae7bb-b4fa-4249-aec4-2a646946ff36'
    # 'status' = 'accepted'
    # 'urls' = {
    #    'resources': [],
    #    'status':
    #    'http://actinia-dev:8088/api/v3/resources/actinia-gdi/
    #    resource_id-cddae7bb-b4fa-4249-aec4-2a646946ff36'}

    return json.loads(resp.text)


def send_binary_cloud_event(event, queue_name, url):
    """Return posted binary event with actinia_job."""
    return send_cloud_event(
        mode="binary",
        version=event["specversion"],
        cetype="com.mundialis.actinia.process.startworker",
        subject=event["subject"],
        actiniaqueuename=queue_name,
        url=url,
    )


def send_structured_cloud_event(event, actinia_job, url):
    """Return posted structured event with actinia_job."""
    # TODO: adjust to queue name
    return send_cloud_event(
        mode="structured",
        version=event["specversion"],
        cetype="com.mundialis.actinia.process.started",
        subject=event["subject"],
        data={"actinia_job": actinia_job},
        url=url,
    )


def send_cloud_event(
    mode="binary",
    version="1.0",
    cetype="com.mundialis.actinia.process.started",
    subject="nc_spm_08/PERMANENT",
    actiniaqueuename=None,
    data="{}",
    url=None,
):
    """Post event and return it."""
    if url is None:
        url = EVENTRECEIVER.url

    attributes = {
        "specversion": version,
        "source": "/actinia-cloudevent-plugin",
        "type": cetype,
        "subject": subject,
        "datacontenttype": "application/json",
        "actiniaqueuename": actiniaqueuename,
    }

    event = CloudEvent(attributes, data)

    # From https://github.com/cloudevents/spec/blob/main/cloudevents/spec.md#message
    # A "structured-mode message" is one where the entire event (attributes and data)
    # are encoded in the message body, according to a specific event format.
    # A "binary-mode message" is one where the event data is stored in the message body,
    # and event attributes are stored as part of message metadata.
    # Often, binary mode is used when the producer of the CloudEvent wishes to add the
    # CloudEvent's metadata to an existing event without impacting the message's body.
    # In most cases a CloudEvent encoded as a binary-mode message will not break an
    # existing receiver's processing of the event because the message's metadata
    # typically allows for extension attributes.
    # In other words, a binary formatted CloudEvent would work for both
    # a CloudEvents enabled receiver as well as one that is unaware of CloudEvents.
    if mode == "binary":
        headers, body = to_binary(event)
    else:
        headers, body = to_structured(event)

    requests.post(url, headers=headers, data=body)

    return event
